#include <edapi/journal/journal.h>
#include <edapi/util.h>

typedef struct {
    gchar *location;
} EDJournalPrivate;

struct _EDJournal {
    GObject parent;
};

struct _EDJournalClass {
    GObjectClass parent_class;
};

G_DEFINE_TYPE_EXTENDED(
    EDJournal,
    ed_journal,
    G_TYPE_OBJECT,
    0,
    G_ADD_PRIVATE(EDJournal)
    );

static void ed_journal_finalize(GObject *obj)
{
    EDJournal *self = ED_JOURNAL(obj);
    EDJournalPrivate *p = ed_journal_get_instance_private(self);

    free(p->location);
    p->location = NULL;

    G_OBJECT_CLASS(ed_journal_parent_class)->finalize(obj);
}

static void ed_journal_class_init(EDJournalClass *klass)
{
    G_OBJECT_CLASS(klass)->finalize = ed_journal_finalize;
}

static EDErrorCode ed_journal_determine_location(EDJournal *self)
{
    EDJournalPrivate *p = ed_journal_get_instance_private(self);
    gchar *location = NULL;

#ifdef G_OS_WIN32
    char const *env = getenv("%USERPROFILE%");

    if (!S_EMPTY(env)) {
        location = g_build_path(
            G_DIR_SEPARATOR_S,
            env,
            "Saved Games",
            "Frontier Developments",
            "Elite Dangerous",
            NULL
            );
    }
#endif

    if (S_EMPTY(location)) {
        return ed_error_invalid;
    }

    if (!g_file_test(location, G_FILE_TEST_IS_DIR)) {
        g_free(location);
        return ed_error_invalid;
    }

    g_free(p->location);
    p->location = location;

    return ed_error_success;
}

static void ed_journal_init(EDJournal *self)
{
    ed_journal_determine_location(self);
}

EDJournal *ed_journal_new(void)
{
    return g_object_new(ED_TYPE_JOURNAL, NULL);
}

gchar const *ed_journal_get_location(EDJournal *self)
{
    return_if_true(self == NULL, NULL);
    EDJournalPrivate *p = ed_journal_get_instance_private(self);
    return p->location;
}

EDErrorCode ed_journal_set_location(EDJournal *self, gchar const *dir)
{
    return_if_true(self == NULL || dir == NULL, ed_error_args);

    EDJournalPrivate *p = ed_journal_get_instance_private(self);

    if (!g_file_test(dir, G_FILE_TEST_IS_DIR)) {
        return ed_error_invalid;
    }

    g_free(p->location);
    p->location = g_strdup(dir);

    return ed_error_success;
}
